# encoding: utf-8

# -------------------------------------------------------------------------------
# (C) Copyright 2019 - 2022 Micro Focus or one of its affiliates.
#
# The only warranties for products and services of Micro Focus and its affiliates
# and licensors (“Micro Focus”) are as may be set forth in the express warranty
# statements accompanying such products and services. Nothing herein should be
# construed as constituting an additional warranty. Micro Focus shall not be
# liable for technical or editorial errors or omissions contained herein. The
# information contained herein is subject to change without notice.
#
# Except as specifically indicated otherwise, this document contains confidential
# information and a valid license is required for possession, use or copying. If
# this work is provided to the U.S. Government, consistent with FAR 12.211 and
# 12.212, Commercial Computer Software, Computer Software Documentation, and
# Technical Data for Commercial Items  are licensed to the U.S. Government under
# vendor's standard commercial license.
# ------------------------------------------------------------------------------
# Copyright (c) 2006 Novell, Inc. All Rights Reserved.
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of version 2 of the GNU General Public License as published by the
# Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, contact Novell, Inc.
#
# To contact Novell about this file by physical or electronic mail, you may find
# current contact information at www.novell.com.
# ------------------------------------------------------------------------------

# File:	modules/NovellNssad.ycp
# Package:	Configuration of novell-nssad
# Summary:	NovellNssad settings, input and output functions
# Authors:	Suresh K. Hosamani <hsuresh@novell.com>
#
# $Id: NovellNssad.ycp 41350 2007-10-10 16:59:00Z dfiser $
#
# Representation of the configuration of novell-nssad.
# Input and output routines.
require "yast"

module Yast
  class NovellNssadClass < Module
    def main
      textdomain "novell-nssad"

      Yast.import "Progress"
      Yast.import "Summary"
      Yast.import "Message"
      Yast.import "Service"
      Yast.import "NovellPatterns"
      Yast.import "NovellUtils"
      Yast.import "NovellAddonInstall"
      Yast.import "NovellLDAP"
      Yast.import "FileUtils"
      Yast.import "Report"
      Yast.import "Popup"
      Yast.import "Nss"
      Yast.import "Mode"
      Yast.import "Stage"
      @Settings = {}
      @oldSettings = {}
      @globals_configured = false

      @xadTestMode = false
      @auto_clone = false
      @isDomainJoined = false

      #Data was modified?
      @modified = false

      @proposal_valid = false

      @upgrade_from = ""

      # Write only, used during autoinstallation.
      # Don't run services and SuSEconfig, it's all done at one place.
      @write_only = false

      # Abort function
      # return boolean return true if abort
      @AbortFunction = fun_ref(method(:Modified), "boolean ()")
    end

    # Abort function
    # @return [Boolean] return true if abort
    def Abort
      return @AbortFunction.call == true if @AbortFunction != nil
      false
    end

    # Data was modified?
    # @return true if modified
    def Modified
      Builtins.y2debug("modified=%1", @modified)
      @modified
    end

    # Mark as modified, for Autoyast.
    def SetModified(value)
      @modified = true

      nil
    end

    def ProposalValid
      @proposal_valid
    end

    def SetProposalValid(value)
      @proposal_valid = value

      nil
    end

    # @return true if module is marked as "write only" (don't start services etc...)
    def WriteOnly
      @write_only
    end

    # Set write_only flag (for autoinstalation).
    def SetWriteOnly(value)
      @write_only = value

      nil
    end


    def SetAbortFunction(function)
      function = deep_copy(function)
      @AbortFunction = deep_copy(function)

      nil
    end

    # StoreCertificate function
    # This function stores ssl certificate into javakeystore to be used by NURM

    def StoreCertificate
      NovellLDAP.Read
      Builtins.y2milestone(
        "NovellNssad: Storing SSL certificate into javakeystore"
      )
      treeName = Ops.get_string(
        NovellLDAP.Settings,
        "CONFIG_LDAP_TREE_NAME",
        ""
      )
      _alias = Ops.add(Ops.add("Storm_", treeName), "_sscert")

      # Novell's Root store  /var/opt/novell/tomcat/conf/cacerts"
      rootCertStore = "/var/opt/novell/tomcat/conf/cacerts"
      cert = "/etc/opt/novell/certs/SSCert.der"

      if !FileUtils.Exists(cert)
        Builtins.y2milestone("Unable to find certificate file %1", cert)
      end

      stored = NovellUtils.StoreTrustedCertificateInJavaKeystore(
        "/etc/opt/novell/certs/SSCert.der",
        rootCertStore,
        _alias
      )
      if !stored
        message = "Unable to add the tree self-signed certificate to the root certificate store"
        Builtins.y2milestone("NovellNssad: %1", message)
      else
        Builtins.y2milestone(
          "NovellNssad: Tree CA Root Certificate was stored in the Novell keystore"
        )
        if Stage.normal
          if Service.Status("novell-tomcat") == 0
            Service.Restart("novell-tomcat")
            Builtins.y2milestone(
              "NovellNssad: novell-tomcat service restarted"
            )
          else
            Service.Start("novell-tomcat")
            Builtins.y2milestone("NovellNssad: novell-tomcat service started")
          end
        end
      end

      nil
    end

    # Read all novell-nssad settings
    # @return true on success
    def Read
      # NovellNssad read dialog caption
      caption = _("Initializing  Configuration")

      # TODO FIXME Set the right number of stages
      steps = 2

      sl = 500
      Builtins.sleep(sl)


      # TODO FIXME Names of real stages
      # We do not set help text here, because it was set outside
      Progress.New(
        caption,
        " ",
        steps,
        [
          # Progress stage 1/3
          _("Read the previous settings"),
          _("Checking dependencies")
        ],
        [
          # Progress step 1/3
          _("Read the previous settings..."),
          _("Checking dependencies..."),
          # Progress finished
          _("Finished")
        ],
        ""
      )
      # read the previous settings
      return false if Abort()
      Progress.NextStage

      # Test fo the upgrade scenario
      upgradeInfo = {}

      @upgrade_from = NovellUtils.UpgradeIsFrom
      if (Mode.update || Mode.autoupgrade) &&  @upgrade_from != ""
        @oldSettings = NovellUtils.GetOldSysconfigMap("novell-nssad")
      end

      Builtins.y2milestone(
        "nssad:
        upgradeFrom = #{@upgrade_from}"
      )

      if !Mode.update && !Mode.autoupgrade
        # Fetching netbios name from cifs.conf to verify domain joined or not

        cmd = "grep -ri \"SERVERNAME\" /etc/opt/novell/cifs/cifs.conf |grep -v ^# |cut -d\" \" -f2"
        val = Convert.to_map(SCR.Execute(path(".target.bash_output"), cmd))
        netbiosname = NovellUtils.CleanUpString(
          Ops.get_string(val, "stdout", "")
        )
        if netbiosname != "" || netbiosname != nil
          cmd2 = Builtins.sformat("klist -k |grep -i %1", netbiosname)
          val2 = Convert.to_map(SCR.Execute(path(".target.bash_output"), cmd2))
          @isDomainJoined = true if Ops.get_integer(val2, "exit", -1) == 0
        end

        service_configured = Convert.to_string(
          SCR.Read(path(".sysconfig.novell-nssad.SERVICE_CONFIGURED"))
        )

        # setting ad_uid_generate_mode to yes / no

        cmd = Builtins.sformat(
          "/usr/bin/nitconfig get ad-uid-generate-mode |cut -d\"=\" -f2|awk '{print $1}'"
        )
        val = Convert.to_map(SCR.Execute(path(".target.bash_output"), cmd))
        ad_uid_generate_mode = false
        if Builtins.tointeger(
            NovellUtils.CleanUpString(Ops.get_string(val, "stdout", ""))
          ) != 0
          ad_uid_generate_mode = true
          Ops.set(@Settings, "AD_UID_GENERATE_MODE", "yes")
        else
          ad_uid_generate_mode = false
          Ops.set(@Settings, "AD_UID_GENERATE_MODE", "no")
        end

        if service_configured != "yes"
          domain_name = NovellUtils.GetDomainName
          domain_name = Builtins.toupper(domain_name)
          Ops.set(@Settings, "CONFIG_NSSAD_DOMAIN_NAME", domain_name)
          Ops.set(@Settings, "CONFIG_NSSAD_ADMIN_NAME", "Administrator")
          Ops.set(@Settings, "CONFIG_NSSAD_CONTEXT", "CN=Computers")
          Ops.set(@Settings, "CONFIG_NSSAD_PRECREATED_OBJECT", "no")
          #Settings["CONFIG_NSSAD_ENABLE_SSHSCP"] = "no";
          Ops.set(@Settings, "CONFIG_NSSAD_ADMIN_GROUP", "Domain Admins")
        else
          Ops.set(
            @Settings,
            "CONFIG_NSSAD_DOMAIN_NAME",
            Convert.to_string(
              SCR.Read(path(".sysconfig.novell-nssad.CONFIG_NSSAD_DOMAIN_NAME"))
            )
          )
          Ops.set(
            @Settings,
            "CONFIG_NSSAD_ADMIN_NAME",
            Convert.to_string(
              SCR.Read(path(".sysconfig.novell-nssad.CONFIG_NSSAD_ADMIN_NAME"))
            )
          )
          Ops.set(
            @Settings,
            "CONFIG_NSSAD_CONTEXT",
            Convert.to_string(
              SCR.Read(path(".sysconfig.novell-nssad.CONFIG_NSSAD_CONTEXT"))
            )
          )
          #		Settings["CONFIG_NSSAD_ENABLE_SSHSCP"] = (string)SCR::Read(.sysconfig.novell-nssad.CONFIG_NSSAD_ENABLE_SSHSCP);
          Ops.set(
            @Settings,
            "CONFIG_NSSAD_PRECREATED_OBJECT",
            Convert.to_string(
              SCR.Read(
                path(".sysconfig.novell-nssad.CONFIG_NSSAD_PRECREATED_OBJECT")
              )
            )
          )

          cmd = Builtins.sformat(
            "/usr/bin/nitconfig get ad-supervisor-group |cut -d\"=\" -f2"
          )
          val = Convert.to_map(SCR.Execute(path(".target.bash_output"), cmd))
          Ops.set(
            @Settings,
            "CONFIG_NSSAD_ADMIN_GROUP",
            NovellUtils.CleanUpString(Ops.get_string(val, "stdout", ""))
          )
        end

        #Read NIT Range setting
        Nss.NitReadSettings
      else
        Ops.set(
          @Settings,
          "CONFIG_NSSAD_DOMAIN_NAME",
          Ops.get_string(@oldSettings, "CONFIG_NSSAD_DOMAIN_NAME", "")
        )
        Ops.set(
          @Settings,
          "CONFIG_NSSAD_ADMIN_NAME",
          Ops.get_string(@oldSettings, "CONFIG_NSSAD_ADMIN_NAME", "")
        )
        Ops.set(
          @Settings,
          "CONFIG_NSSAD_CONTEXT",
          Ops.get_string(@oldSettings, "CONFIG_NSSAD_CONTEXT", "")
        )
        #		Settings["CONFIG_NSSAD_ENABLE_SSHSCP"] = (string)SCR::Read(.sysconfig.novell-nssad.CONFIG_NSSAD_ENABLE_SSHSCP);
        Ops.set(
          @Settings,
          "CONFIG_NSSAD_PRECREATED_OBJECT",
          Ops.get_string(@oldSettings, "CONFIG_NSSAD_PRECREATED_OBJECT", "")
        )
      end

      return false if Abort()
      Progress.NextStage

      @globals_configured = true

      Progress.Title(_("Finished"))
      Builtins.sleep(sl)

      return false if Abort()
      @modified = false
      true
    end

    # Write all novell-nssad settings
    # @return true on success
    def Write

      if !Ops.get(
          NovellAddonInstall.products_to_configure,
          "novell-nssad",
          true
        )
        Builtins.y2milestone("NovellNssad -- configure later")
        return true
      end

      Builtins.y2milestone("novell-nssad::Write started")

      # NovellNssad read dialog caption
      caption = _("Saving OES Storage Services AD Support Configuration")

      # TODO FIXME And set the right number of stages
      steps = 2

      sl = 500
      Builtins.sleep(sl)

      # TODO FIXME Names of real stages
      # We do not set help text here, because it was set outside
      Progress.New(
        caption,
        " ",
        steps,
        [
          # Progress stage 1/1
          _("Write the settings")
        ],
        [
          # Progress step 1/1
          _("Writing the settings..."),
          # Progress finished
          _("Finished")
        ],
        ""
      )

      if !Mode.update && !Mode.autoupgrade
        if NovellPatterns.IsPatternInstalled("novell-domain-services") == true
          Report.Error(
            Builtins.sformat(
              _(
                "Domain join cannot be performed when OES Domain Services for Windows pattern is installed"
              )
            )
          )
          return true
        end

        pattern = "novell-nssad"
        if NovellPatterns.IsPatternInstalled(pattern) == false
          Builtins.y2milestone(
            "The required pattern, %1, is not installed.",
            pattern
          )
          if Mode.normal
            Report.Error(
              Builtins.sformat(
                _(
                  "Configuration aborted. Install the required %1 pattern and try again."
                ),
                pattern
              )
            )
          end

          return true
        end


        if Service.Status("novell-cifs") != 0
          Report.Error(
            Builtins.sformat(_("Ensure that the CIFS service is running."))
          )
          return false
        end

        Ops.set(
          @Settings,
          "CONFIG_NSSAD_DOMAIN_NAME",
          Builtins.toupper(
            Ops.get_string(@Settings, "CONFIG_NSSAD_DOMAIN_NAME", "")
          )
        )

        # If Domain is already joined skipping domain join operation

        if !@isDomainJoined
          Popup.ShowFeedback(
            _("Domain Join is in progress..."),
            _("This may take a while.")
          )

          cmd2 = Builtins.sformat(
            "/opt/novell/xad/bin/kinit -E %1@%2",
            Ops.get_string(@Settings, "CONFIG_NSSAD_ADMIN_NAME", ""),
            Ops.get_string(@Settings, "CONFIG_NSSAD_DOMAIN_NAME", "")
          )
          cmd2 = Builtins.sformat(
            "ADM_PASSWD='%1' %2",
            Ops.get_string(@Settings, "CONFIG_NSSAD_ADMIN_PASSWORD", ""),
            cmd2
          )
          val = Convert.to_map(SCR.Execute(path(".target.bash_output"), cmd2))
          exit = Ops.get_integer(val, "exit", -1)
          Builtins.y2milestone("map output %1", val)
          if exit != 0
            Report.Error(
              _(
                "kinit has failed, and the AD configuration has been aborted.\nRetry with the correct AD configuration details."
              )
            )
            Builtins.y2milestone(
              "kinit has failed, and the AD configuration has been aborted. Retry with the correct AD configuration details."
            )
	    UI.CloseDialog
            return true
          end
          cmd2 = Builtins.sformat(
            "/opt/novell/bin/novell-ad-util --join --domain-name '%1' --service-principal cifs --context '%2' --pre-created-object %3",
            Ops.get_string(@Settings, "CONFIG_NSSAD_DOMAIN_NAME", ""),
            Ops.get_string(@Settings, "CONFIG_NSSAD_CONTEXT", ""),
            Ops.get_string(@Settings, "CONFIG_NSSAD_PRECREATED_OBJECT", "")
          )
          val = Convert.to_map(SCR.Execute(path(".target.bash_output"), cmd2))
          exit = Ops.get_integer(val, "exit", -1)
          if exit == 0
            Builtins.y2milestone("Successfully joined to the domain.")
          elsif exit == 7
            Report.Error(
              Builtins.sformat(
                _("The OES server is already part of the AD Domain.")
              )
            )
            Builtins.y2milestone(
              "The OES server is already part of the AD Domain."
            )
            return true
          elsif exit == 6
            Report.Error(
              Builtins.sformat(
                _("The pre-created computer object does not exist.\n"+
                  "Create the computer object on the AD server and retry configuring NSS AD."
                 )
              )
            )
            Builtins.y2milestone(
              "The pre-created computer object does not exist. Create the computer object on the AD server and retry configuring NSS AD."
            )
            return true
          elsif exit == 14
            Report.Error(
              Builtins.sformat(
                _("The OES computer object already exists on the AD server.")
              )
            )
            Builtins.y2milestone(
              "The OES computer object already exists on the AD server."
            )
            return true
          elsif exit == 15
            Report.Error(
              Builtins.sformat(
                _(
                  "The pre-created OES computer object does not exist in\nthe specified container (%1)."
                ),
                Ops.get_string(@Settings, "CONFIG_NSSAD_CONTEXT", "")
              )
            )
            Builtins.y2milestone(
              "The pre-created OES computer object does not exist in the specified container (%1)",
              Ops.get_string(@Settings, "CONFIG_NSSAD_CONTEXT", "")
            )
            return true
          else
            Report.Error(
              _(
                "Domain join operation has failed, and the AD configuration has been aborted.\nRetry with the correct AD configuration details."
              )
            )
            Builtins.y2milestone(
              "Domain join operation has failed, and the AD configuration has been aborted. Retry with the correct AD configuration details."
            )
            return true
          end

          Popup.ClearFeedback


          # write settings
          return false if Abort()
          Progress.NextStage
        end
        #apply settings
        SCR.Write(path(".sysconfig.novell-nssad.SERVICE_CONFIGURED"), "no")

        SCR.Write(
          path(".sysconfig.novell-nssad.CONFIG_NSSAD_DOMAIN_NAME"),
          Ops.get_string(@Settings, "CONFIG_NSSAD_DOMAIN_NAME", "")
        )
        SCR.Write(
          path(".sysconfig.novell-nssad.CONFIG_NSSAD_ADMIN_NAME"),
          Ops.get_string(@Settings, "CONFIG_NSSAD_ADMIN_NAME", "")
        )
        SCR.Write(
          path(".sysconfig.novell-nssad.CONFIG_NSSAD_CONTEXT"),
          Ops.get_string(@Settings, "CONFIG_NSSAD_CONTEXT", "")
        )
        SCR.Write(
          path(".sysconfig.novell-nssad.CONFIG_NSSAD_PRECREATED_OBJECT"),
          Ops.get_string(@Settings, "CONFIG_NSSAD_PRECREATED_OBJECT", "")
        )
        #	SCR::Write(.sysconfig.novell-nssad.CONFIG_NSSAD_ENABLE_SSHSCP,Settings["CONFIG_NSSAD_ENABLE_SSHSCP"]:"");

        #if enable_ssh_scp is selected in the gui

        # if(Settings["CONFIG_NSSAD_ENABLE_SSHSCP"]:"" == "yes")
        #     {
        #         string domain_name = Settings["CONFIG_NSSAD_DOMAIN_NAME"]:"";
        #         list <string> split_domain_name = splitstring(domain_name,".");
        #         if(size(split_domain_name) > 0)
        #         domain_name = mergestring(split_domain_name,",dc=");
        #         domain_name= "dc=" + domain_name;
        # 		string cmd = sformat("/opt/novell/nit/bin/ssh-scp-config.sh -i %1 -d %2",Settings["CONFIG_NSSAD_DNS_NAME"]:"",domain_name);
        #     	map val = (map)SCR::Execute(.target.bash_output, cmd);
        # 		y2milestone("cmd is %1",cmd);
        # 		y2milestone("map is %1",val);
        # 		if(Service::Status("nscd") != 0)
        # 		{
        # 			Service::Start("nscd");
        # 			if(Service::Status("nscd") == 0)
        # 				y2milestone("Started nscd service");
        # 			else
        # 				y2milestone("Starting nscd service failed");
        # 		}
        # 		else
        # 		{
        # 			Service::Restart("nscd");
        # 			if(Service::Status("nscd") == 0)
        # 				y2milestone("Restarted nscd service");
        # 			else
        # 				y2milestone("Restarting nscd service failed");
        # 		}
        #     }
        #     else
        # 	{
        #         string domain_name = Settings["CONFIG_NSSAD_DOMAIN_NAME"]:"";
        #         list <string> split_domain_name = splitstring(domain_name,".");
        #         if(size(split_domain_name) > 0)
        #         domain_name = mergestring(split_domain_name,",dc=");
        #         domain_name= "dc=" + domain_name;
        # 		string cmd = sformat("/opt/novell/nit/bin/ssh-scp-deconfig.sh -i %1 -d %2",Settings["CONFIG_NSSAD_DNS_NAME"]:"",domain_name);
        #     	map val = (map)SCR::Execute(.target.bash_output, cmd);
        # 		y2milestone("nssad:executing %1",cmd);
        # 		y2milestone("nssad:val is %1",val);
        # 		cmd= sformat("/usr/bin/nitconfig set ad-uid-generate-mode=0");
        #     	val = (map)SCR::Execute(.target.bash_output, cmd);
        # 		y2milestone("NovellNssad:executing %1",cmd);
        # 		y2milestone("NovellNssad:output %1",val);
        # 	}
        if Ops.get_string(@Settings, "AD_UID_GENERATE_MODE", "") == "yes"
          #Setting NIT Range
          Nss.NitRangeSettings
          cmd2 = Builtins.sformat(
            "/usr/bin/nitconfig set ad-uid-generate-mode=1"
          )
          val = Convert.to_map(SCR.Execute(path(".target.bash_output"), cmd2))
          Builtins.y2milestone("NovellNssad:executing %1", cmd2)
          Builtins.y2milestone("NovellNssad:output %1", val)
        else
          cmd2 = Builtins.sformat(
            "/usr/bin/nitconfig set ad-uid-generate-mode=0"
          )
          val = Convert.to_map(SCR.Execute(path(".target.bash_output"), cmd2))
          Builtins.y2milestone("NovellNssad:executing %1", cmd2)
          Builtins.y2milestone("NovellNssad:output %1", val)
        end

        if !@isDomainJoined
          cmd2 = Builtins.sformat(
            "/usr/bin/nitconfig add -a -g %1 -j %2",
            Ops.get_string(@Settings, "CONFIG_NSSAD_DOMAIN_NAME", ""),
            Ops.get_string(@Settings, "CONFIG_NSSAD_DOMAIN_NAME", "")
          )
          val = Convert.to_map(SCR.Execute(path(".target.bash_output"), cmd2))
          Builtins.y2milestone("cmd executed is %1", cmd2)
          Builtins.y2milestone("val is %1", val)
        end

        if @xadTestMode == false
          Service.Enable("novell-nit")
          Service.Restart("novell-nit")
          Builtins.y2milestone("novell-nit service restarted")
        end
        if !@isDomainJoined
          cmd2 = Builtins.sformat(
            "/usr/bin/nitconfig set ad-supervisor-group='%1'",
            Ops.get_string(@Settings, "CONFIG_NSSAD_ADMIN_GROUP", "")
          )
          val = Convert.to_map(SCR.Execute(path(".target.bash_output"), cmd2))
          Builtins.y2milestone("cmd executed is %1", cmd2)
          Builtins.y2milestone("val is %1", val)
        end
        # reloading syslog

        cmd = "systemctl restart rsyslog.service"

        ret = Convert.to_map(SCR.Execute(path(".target.bash_output"), cmd))
        Builtins.y2milestone(
          "NovellNssad.Write:executing:%1, Returned: %2",
          cmd,
          ret
        )

        SCR.Write(path(".sysconfig.novell-nssad.SERVICE_CONFIGURED"), "yes")

        # Calling to store ssl certificate into javastore to be used by NURM
        StoreCertificate()

        # Flush the sysconfig file.
        SCR.Write(path(".sysconfig.novell-nssad"), nil)
      end

      if Mode.update || Mode.autoupgrade
        SCR.Write(
          path(".sysconfig.novell-nssad.CONFIG_NSSAD_DOMAIN_NAME"),
          Ops.get_string(@Settings, "CONFIG_NSSAD_DOMAIN_NAME", "")
        )
        SCR.Write(
          path(".sysconfig.novell-nssad.CONFIG_NSSAD_ADMIN_NAME"),
          Ops.get_string(@Settings, "CONFIG_NSSAD_ADMIN_NAME", "")
        )
        SCR.Write(
          path(".sysconfig.novell-nssad.CONFIG_NSSAD_CONTEXT"),
          Ops.get_string(@Settings, "CONFIG_NSSAD_CONTEXT", "")
        )
        SCR.Write(
          path(".sysconfig.novell-nssad.CONFIG_NSSAD_PRECREATED_OBJECT"),
          Ops.get_string(@Settings, "CONFIG_NSSAD_PRECREATED_OBJECT", "")
        )
        SCR.Write(path(".sysconfig.novell-nssad.SERVICE_CONFIGURED"), "yes")
        SCR.Write(path(".sysconfig.novell-nssad"), nil)
      end
      return false if Abort()

      # Progress finished
      Progress.NextStage
      Progress.Finish
      Builtins.sleep(sl)

      return false if Abort()
      true
    end

    # Get all novell-nssad settings from the first parameter
    # (For use by autoinstallation.)
    # @param [Hash] settings The YCP structure to be imported.
    # @return [Boolean] True on success
    def Import(settings)
      settings = deep_copy(settings)
      # TODO FIXME: your code here (fill the above mentioned variables)...
      if settings == {}
        @globals_configured = false
        Ops.set(NovellAddonInstall.products_to_configure, "novell-nssad", false)
        return true
      end

      Ops.set(
        @Settings,
        "CONFIG_NSSAD_DOMAIN_NAME",
        Ops.get_string(settings, "domain_name", "")
      )
      Ops.set(
        @Settings,
        "CONFIG_NSSAD_ADMIN_NAME",
        Ops.get_string(settings, "admin_name", "")
      )
      Ops.set(
        @Settings,
        "CONFIG_NSSAD_ADMIN_PASSWORD",
        Ops.get_string(settings, "admin_password", "")
      )
      Ops.set(
        @Settings,
        "CONFIG_NSSAD_CONTEXT",
        Ops.get_string(settings, "ad_context", "")
      )
      Ops.set(
        @Settings,
        "CONFIG_NSSAD_PRECREATED_OBJECT",
        Ops.get_string(settings, "pre_created_object", "")
      )
      Ops.set(
        Nss.Settings_nit,
        "CONFIG_NIT_START_RANGE",
        Ops.get_integer(settings, "nit_start_range", 100000)
      )
      Ops.set(
        Nss.Settings_nit,
        "CONFIG_NIT_END_RANGE",
        Ops.get_integer(settings, "nit_end_range", 200000)
      )
      Ops.set(
        @Settings,
        "AD_UID_GENERATE_MODE",
        Ops.get_string(settings, "ad_uid_generate_mode", "")
      )
      # Settings["CONFIG_NSSAD_ENABLE_SSHSCP"] = settings["enable_ssh_scp"]:"";
      # 	if(Settings["AD_UID_GENERATE_MODE"]:"" == "yes" && Settings["CONFIG_NSSAD_ENABLE_SSHSCP"]:"" == "yes")
      # 	{
      # 		Settings["CONFIG_NSSAD_ENABLE_SSHSCP"] = "no";
      # 		y2milestone("Both ad_uid_generate_mode and enable_ssh_scp cannot be set to yes, unsetting enable_ssh_scp to no");
      # 	}
      Ops.set(
        @Settings,
        "CONFIG_NSSAD_ADMIN_GROUP",
        Ops.get_string(settings, "domain_admin_group", "")
      )
      @globals_configured = true
      true
    end

    # Dump the novell-nssad settings to a single map
    # (For use by autoinstallation.)
    # @return [Hash] Dumped settings (later acceptable by Import ())
    def Export
      # TODO FIXME: your code here (return the above mentioned variables)...
      settings = {}

      return settings if NovellUtils.CloneSettingstoAutoinstXml("novell-nssad") == false

      if @auto_clone
        Ops.set(NovellAddonInstall.products_to_configure, "novell-nssad", true)
      end
      if Ops.get(NovellAddonInstall.products_to_configure, "novell-nssad", true)
        Ops.set(
          settings,
          "domain_name",
          Ops.get_string(@Settings, "CONFIG_NSSAD_DOMAIN_NAME", "")
        )
        Ops.set(
          settings,
          "admin_name",
          Ops.get_string(@Settings, "CONFIG_NSSAD_ADMIN_NAME", "")
        )
        Ops.set(settings, "admin_password", NovellUtils.passwordMessage)
        Ops.set(
          settings,
          "ad_context",
          Ops.get_string(@Settings, "CONFIG_NSSAD_CONTEXT", "")
        )
        Ops.set(
          settings,
          "pre_created_object",
          Ops.get_string(@Settings, "CONFIG_NSSAD_PRECREATED_OBJECT", "")
        )
        Ops.set(
          settings,
          "nit_start_range",
          Ops.get_integer(Nss.Settings_nit, "CONFIG_NIT_START_RANGE", 100000)
        )
        Ops.set(
          settings,
          "nit_end_range",
          Ops.get_integer(Nss.Settings_nit, "CONFIG_NIT_END_RANGE", 200000)
        )
        Ops.set(
          settings,
          "ad_uid_generate_mode",
          Ops.get_string(@Settings, "AD_UID_GENERATE_MODE", "")
        )
        #	settings["enable_ssh_scp"] = Settings["CONFIG_NSSAD_ENABLE_SSHSCP"]:"";
        Ops.set(
          settings,
          "domain_admin_group",
          Ops.get_string(@Settings, "CONFIG_NSSAD_ADMIN_GROUP", "")
        )
      end
      deep_copy(settings)
    end

    # Dump the novell-nssad settings to a single map
    # (For use by autoinstallation.)
    # @return [Hash] Dumped settings (later acceptable by Import ())
    def ExportSettings
      # TODO FIXME: your code here (return the above mentioned variables)...
      settings = {}

      return settings if NovellUtils.CloneSettingstoAutoinstXml("novell-nssad") == false

      if @auto_clone
        Ops.set(NovellAddonInstall.products_to_configure, "novell-nssad", true)
      end
      if Ops.get(NovellAddonInstall.products_to_configure, "novell-nssad", true)
        Ops.set(
          settings,
          "domain_name",
          Ops.get_string(@Settings, "CONFIG_NSSAD_DOMAIN_NAME", "")
        )
        Ops.set(
          settings,
          "admin_name",
          Ops.get_string(@Settings, "CONFIG_NSSAD_ADMIN_NAME", "")
        )
        Ops.set(
          settings,
          "admin_password",
          Ops.get_string(@Settings, "CONFIG_NSSAD_ADMIN_PASSWORD", "")
        )
        Ops.set(
          settings,
          "pre_created_object",
          Ops.get_string(@Settings, "CONFIG_NSSAD_PRECREATED_OBJECT", "")
        )
        Ops.set(
          settings,
          "ad_context",
          Ops.get_string(@Settings, "CONFIG_NSSAD_CONTEXT", "")
        )
        Ops.set(
          settings,
          "nit_start_range",
          Ops.get_integer(Nss.Settings_nit, "CONFIG_NIT_START_RANGE", 100000)
        )
        Ops.set(
          settings,
          "nit_end_range",
          Ops.get_integer(Nss.Settings_nit, "CONFIG_NIT_END_RANGE", 200000)
        )
        Ops.set(
          settings,
          "ad_uid_generate_mode",
          Ops.get_string(@Settings, "AD_UID_GENERATE_MODE", "")
        )
        Ops.set(
          settings,
          "domain_admin_group",
          Ops.get_string(@Settings, "CONFIG_NSSAD_ADMIN_GROUP", "")
        ) 
        #  settings["enable_ssh_scp"] = Settings["CONFIG_NSSAD_ENABLE_SSHSCP"]:"";
      end
      deep_copy(settings)
    end

    # Tranlate a yes or no string
    # @return translated "yes" or "no" string
    def Trans_yes_no(yesno)
      if Builtins.tolower(yesno) == "yes"
        return _("yes")
      else
        return _("no")
      end
    end


    # Create a textual summary and a list of unconfigured cards
    # @return summary of the current configuration
    def Summary
      # TODO FIXME: your code here...
      # Configuration summary text for autoyast
      summary = ""

      if !@globals_configured
        nc = Summary.NotConfigured
        summary = Summary.AddLine(summary, nc)
      else
        summary = Summary.OpenList(summary)

        if Mode.update || Mode.autoupgrade
          summary = Summary.AddListItem(
            summary,
            _("Upgrading from earlier version of OES: yes")
          )
        end
        summary = Summary.AddListItem(
          summary,
          Builtins.sformat(
            _("AD Domain Name: %1"),
            Ops.get_string(@Settings, "CONFIG_NSSAD_DOMAIN_NAME", "")
          )
        )
        if !Mode.update && !Mode.autoupgrade
          summary = Summary.AddListItem(
            summary,
            Builtins.sformat(
              _("AD Supervisor Group: %1"),
              Ops.get_string(@Settings, "CONFIG_NSSAD_ADMIN_GROUP", "")
            )
          )
        end
        summary = Summary.AddListItem(
          summary,
          Builtins.sformat(
            _("AD Administrator Name: %1"),
            Ops.get_string(@Settings, "CONFIG_NSSAD_ADMIN_NAME", "")
          )
        )
        summary = Summary.AddListItem(
          summary,
          Builtins.sformat(
            _("Container to create Computer Object: %1"),
            Ops.get_string(@Settings, "CONFIG_NSSAD_CONTEXT", "")
          )
        )
        summary = Summary.AddListItem(
          summary,
          Builtins.sformat(
            _("Use pre-created computer object: %1"),
           Trans_yes_no(Ops.get_string(@Settings, "CONFIG_NSSAD_PRECREATED_OBJECT", ""))
          )
        )
        summary = Summary.AddListItem(
          summary,
          Builtins.sformat(
            _("Generate UIDs for AD users: %1"),
           Trans_yes_no(Ops.get_string(@Settings, "AD_UID_GENERATE_MODE", ""))
          )
        )
        #		summary = Summary::AddListItem(summary, sformat(_("Enable SSH/SCP for AD users: %1"), Settings["CONFIG_NSSAD_ENABLE_SSHSCP"]:"") );
        summary = Summary.AddListItem(
          summary,
          Builtins.sformat(
            _("UID start range: %1"),
            Ops.get_integer(Nss.Settings_nit, "CONFIG_NIT_START_RANGE", 100000)
          )
        )
        summary = Summary.AddListItem(
          summary,
          Builtins.sformat(
            _("UID end range: %1"),
            Ops.get_integer(Nss.Settings_nit, "CONFIG_NIT_END_RANGE", 200000)
          )
        )

        summary = Summary.CloseList(summary)
      end
      [summary, []]
    end

    # Create an overview table with all configured cards
    # @return table items
    def Overview
      # TODO FIXME: your code here...
      []
    end

    # Return packages needed to be installed and removed during
    # Autoinstallation to insure module has all needed software
    # installed.
    # @return [Hash] with 2 lists.
    def AutoPackages
      # TODO FIXME: your code here...
      { "install" => [], "remove" => [] }
    end

    publish :variable => :Settings, :type => "map"
    publish :variable => :oldSettings, :type => "map"
    publish :variable => :globals_configured, :type => "boolean"
    publish :variable => :xadTestMode, :type => "boolean"
    publish :variable => :auto_clone, :type => "boolean"
    publish :variable => :isDomainJoined, :type => "boolean"
    publish :function => :Modified, :type => "boolean ()"
    publish :function => :Abort, :type => "boolean ()"
    publish :function => :SetModified, :type => "void (boolean)"
    publish :function => :ProposalValid, :type => "boolean ()"
    publish :function => :SetProposalValid, :type => "void (boolean)"
    publish :function => :WriteOnly, :type => "boolean ()"
    publish :function => :SetWriteOnly, :type => "void (boolean)"
    publish :function => :SetAbortFunction, :type => "void (boolean ())"
    publish :function => :StoreCertificate, :type => "void ()"
    publish :function => :Read, :type => "boolean ()"
    publish :function => :Write, :type => "boolean ()"
    publish :function => :Import, :type => "boolean (map)"
    publish :function => :Export, :type => "map ()"
    publish :function => :ExportSettings, :type => "map ()"
    publish :function => :Summary, :type => "list ()"
    publish :function => :Overview, :type => "list ()"
    publish :function => :AutoPackages, :type => "map ()"
  end

  NovellNssad = NovellNssadClass.new
  NovellNssad.main
end
